/**
 * gclc:net.bigeon.gclc.command.CommandParameters.java
 * Created on: Dec 24, 2014
 */
package net.bigeon.gclc.command;

/*-
 * #%L
 * Generic Command Ligne console
 * %%
 * Copyright (C) 2014 - 2018 bigeon.fr
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import net.bigeon.gclc.exception.CommandParsingException;

/** An object representing a collection of parameters.
 * <p>
 * It is used for defaulting values.
 *
 * @author Emmanuel BIGEON */
public final class CommandParameters {
    /** Number of element for a string argument. */
    private static final int           STRINGARG_NUMBER_OF_ELEMENTS = 2;
    /** Boolean arguments. */
    private final Map<String, Boolean> booleanArguments;
    /** String arguments. */
    private final Map<String, String>  stringArguments;
    /** Arguments restriction on the named ones. */
    private final boolean              strict;
    /** additional (unnamed) parameters. */
    private final List<String>         additional                   = new ArrayList<>();

    /** Create a command parameter object.
     *
     * @param bools the boolean parameters
     * @param strings the string parameters
     * @param strict if the argument are restricted to the declared ones */
    public CommandParameters(final Set<String> bools, final Set<String> strings,
                             final boolean strict) {
        booleanArguments = new ConcurrentHashMap<>(bools.size());
        for (final String string : bools) {
            booleanArguments.put(string, Boolean.FALSE);
        }
        // Cannot use concurrent because of the null values.
        stringArguments = new HashMap<>(strings.size());
        for (final String string : strings) {
            stringArguments.put(string, null);
        }
        this.strict = strict;
    }

    /** Get the value of a string argument.
     *
     * @param key the key
     * @return the associated value, null if it was not specified */
    public String get(final String key) {
        return stringArguments.get(key);
    }

    /** Get the additional (unrecognized) arguments.
     *
     * @return additional non parsed parameters */
    public List<String> getAdditionals() {
        return Collections.unmodifiableList(additional);
    }

    /** Get the value of a boolean argument.
     *
     * @param key the key
     * @return if the key was specified */
    public boolean isActive(final String key) {
        Boolean val = booleanArguments.get(key);
        return val != null && val.booleanValue();
    }

    /** Get the boolean arguments.
     *
     * @return the boolean arguments */
    public Set<String> getBooleanArgumentKeys() {
        return booleanArguments.keySet();
    }

    /** Get the string arguments.
     *
     * @return the boolean arguments */
    public Set<String> getStringArgumentKeys() {
        return stringArguments.keySet();
    }

    /** Test if an argument exists in this object.
     *
     * @param key the key
     * @return if the key is present in string arguments or boolean ones. */
    public boolean hasArgument(final String key) {
        return stringArguments.containsKey(key) || booleanArguments.containsKey(key);
    }

    /** Attempt to parse an argument.
     * <p>
     * This method return 0 if the parsing was incorrect, or the number of parsed
     * elements.
     *
     * @param arg the argument
     * @param next the next element
     * @return the number of element read */
    private int parseArg(final String arg, final String next) {
        if (!arg.startsWith("-")) { //$NON-NLS-1$
            if (strict) {
                return 0;
            }
            additional.add(arg);
            return 1;
        }
        final String name = arg.substring(1);
        if (booleanArguments.containsKey(name)) {
            booleanArguments.put(name, Boolean.TRUE);
            return 1;
        }
        if (stringArguments.containsKey(name)) {
            return parseStringArg(name, next);
        }
        if (strict) {
            return 0;
        }
        additional.add(name);
        return 1;
    }

    /** Parse arguments.
     *
     * @param args the arguments to parse
     * @throws CommandParsingException if the arguments parsing failed */
    public void parseArgs(final String... args) throws CommandParsingException {
        int i = 0;
        while (i < args.length) {
            String next = null;
            if (i < args.length - 1) {
                next = args[i + 1];
            }
            final int p = parseArg(args[i], next);
            if (p == 0) {
                throw new CommandParsingException("Invalid parameter " + args[i]); //$NON-NLS-1$
            }
            i += p;
        }

    }

    /** Add a string arg value.
     *
     * @param name the string arg name
     * @param next the string arg value
     * @return 2 or 0 if next is invalid */
    private int parseStringArg(final String name, final String next) {
        if (next == null) {
            return 0;
        }
        stringArguments.put(name, next);
        return STRINGARG_NUMBER_OF_ELEMENTS;
    }

    /** Set a boolean parameter value.
     *
     * @param string the key
     * @param value the value */
    public void set(final String string, final boolean value) {
        if (booleanArguments.containsKey(string)) {
            booleanArguments.put(string, Boolean.valueOf(value));
        }
    }

    /** Set a string parameter value.
     *
     * @param string the key
     * @param value the value */
    public void set(final String string, final String value) {
        if (stringArguments.containsKey(string)) {
            stringArguments.put(string, value);
        }
    }
}
